package com.uc.base.net;

import com.uc.base.net.metrics.IHttpConnectionMetrics;

public interface IHttpClientAsync {

    /**
     * enqueue the http request in normal priority. Make sure method {@link HttpClientAsync#getRequest(String url)} has
     * called when start send request
     */
    void sendRequest(IRequest request);

    void setProxy(String url, int port);

    /**
     * enqueue the http request width priority. Make sure method {@link HttpClientAsync#getRequest(String url)} has
     * called when start send request
     *
     * @param highPriority
     *            true, the request will add in request queue's head, otherwise, add queue's end
     */
    void sendRequest(IRequest request, boolean highPriority);

    void cancel(IRequest request);

    IHttpConnectionMetrics getHttpConnectionMetrics();


    void setDataDecompressEventListener(IDataDecompressEventListener listener);

    IDataDecompressEventListener getDataDecompressEventListener();

    /**
     * set connection timeout , unit : ms
     *
     * @param timeout
     */
    void setConnectionTimeout(int timeout);

    void setRequestTimeout(int timeout);

    /**
     * set socket timeout , unit : ms
     *
     * @param timeout
     */
    void setSocketTimeout(int timeout);

    /**
     * user name and password will be use when web server need an authentication
     *
     * @param username
     * @param password
     */
    void setAuth(String username, String password);

    /**
     * get the {@link com.uc.base.net.IRequest}, and setup the request properties before start a network request
     *
     * @return {@link com.uc.base.net.IRequest}
     */
    IRequest getRequest(String url) throws IllegalArgumentException;

    /**
     * set HTTP connection metrics
     * @param tag indicate which metrics belongs to.
     */
    void setMetricsTAG(String tag);
}
