package com.uc.base.net;

import com.uc.base.net.metrics.IHttpConnectionMetrics;

public interface IHttpClientSync {

    IResponse sendRequest(IRequest request);

    void close();

    int errorCode();

    void followRedirects(boolean follow);
    
    IHttpConnectionMetrics getHttpConnectionMetrics();

    void recycle(int keepTime);

    /**
     * set connection timeout , unit : ms
     *
     * @param timeout
     */
    void setConnectionTimeout(int timeout);

    void setRequestTimeout(int timeout);

    /**
     * set socket timeout , unit : ms
     *
     * @param timeout
     */
    void setSocketTimeout(int timeout);

    /**
     * user name and password will be use when web server need an authentication
     *
     * @param username
     * @param password
     */
    void setAuth(String username, String password);

    /**
     * set the proxy host
     *
     * @param url
     *            the proxy url, for example http://www.ucweb.com
     * @param port
     *            the proxy port, for example 8080
     */
    void setProxy(String url, int port);

    void setFollowProxy(boolean followProxy);

    /**
     * get the {@link com.uc.base.net.IRequest}, and setup the request properties before start a network request
     *
     * @return {@link com.uc.base.net.IRequest}
     */
    IRequest getRequest(String url) throws IllegalArgumentException;

    /**
     * set HTTP connection metrics
     * @param tag indicate which metrics belongs to.
     */
    void setMetricsTAG(String tag);
}
