/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-3
 *
 * Description : 
 *
 * Creation    : 2014-3-3
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-3, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net;

//import com.uc.base.net.adaptor.Connection;
import com.uc.base.net.adaptor.Headers.Header;

import java.io.InputStream;
import java.util.ArrayList;

/**
 * http request
 */
public interface IRequest {
    
    /**
     * get request url
     * 
     * @return
     */
    public String getUrl();
    
    /**
     * set request method, {@link com.uc.base.net.adaptor.Headers#METHOD_GET} {@link com.uc.base.net.adaptor.Headers#METHOD_POST}
     * <p>
     * throw IllegalArgument exception if method is null or invalide
     * 
     * @param method
     */
    public void setMethod(String method);
    
    /**
     * get current request method
     * 
     * @return
     */
    public String getMethod();
    
    /**
     * add request properties
     * 
     * @param key
     * @param value
     */
    public void addHeader(String key, String value);
    
    /**
     * Replaces the first occurence of the header with the same name. If no header with the same name is found the given
     * header is added to the end of the list.
     * 
     * @param header
     *            the new header that should replace the first header with the same name if present in the list.
     */
    public void updateHeader(String key, String value);
    
    /**
     * Replaces the first occurence of the header with the same name. If no header with the same name is found the given
     * header is added to the end of the list.
     * 
     * @param header
     *            the new header that should replace the first header with the same name if present in the list.
     */
    public void updateHeader(Header header);
    
    /**
     * add headers
     * 
     * @param headers
     */
    public void addHeaders(ArrayList<Header> headers);
    
    /**
     * set request content, the input stream and read length
     * 
     * @param bodyProvider
     * @param bodyLength
     */
    public void setBodyProvider(InputStream bodyProvider, long bodyLength);
    
    /**
     * set request content, a file path, only support absolute path and file://path throw IllegalArgumentException if
     * file is't exist
     * 
     * @param pathUrl
     */
    public void setBodyProvider(String pathUrl);
    
    /**
     * set request content, byte array
     * 
     * @param bodyProvider
     */
    public void setBodyProvider(byte[] bodyProvider);
    
    /**
     * get all of headers
     * 
     * @return
     */
    public Header[] getAllHeaders();
    
    /**
     * remove request header, it will remove all header with the same name,
     * <p>
     * for example removeHeaders("Set-Cookies"), will remove all cookies
     * 
     * @param header
     */
    public void removeHeaders(String header);
    
    /**
     *  Removes the given header.
     * 
     * @param header
     *            the header to remove
     */
    public void removeHeader(Header header);
    
    /**
     * Gets all of the headers with the given name. The returned array maintains the relative order in which the headers
     * were added.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header(s) to get
     * 
     * @return an array of length >= 0
     */
    public Header[] getHeaders(String header);
    
    /**
     * 
     * @param header
     * @return
     */
    public boolean containsHeaders(String header);
    
    /**
     * set accept encoding
     * 
     * @param string
     */
    public void setAcceptEncoding(String string);
    
    /**
     * set content type
     * 
     * @param string
     */
    public void setContentType(String string);

    void recordUseBackupDnsIp();

    boolean isUseBackupDnsIp();

    boolean setZstdSupport(boolean isSupport);

    boolean isZstdSupport();

    /*
     *  ignore SSL error if occurs.
     */
    void ignoreSSLErrorIfOccurs();

    void disableHttp2();

    void setCookieEnable(boolean enable);
}
