/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-3
 *
 * Description : 
 *
 * Creation    : 2014-3-3
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-3, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net;

import com.uc.base.net.adaptor.Headers.Header;

import java.io.IOException;
import java.io.InputStream;

public interface IResponse {

    String PROTOCOL_HTTP1 = "HTTP1.1";
    String PROTOCOL_HTTP2 = "HTTP2.0";
    
    /**
     * remote address
     */
    public String getRemoteAddress();
    
    /**
     * get remote host name
     * 
     * @return
     */
    public String getRemoteHostName();
    
    /**
     * get remote port
     * 
     * @return
     */
    public int getRemotePort();
    
    /**
     * Sets the response status line (like "HTTP/1.0 200 OK") or request line (like "GET / HTTP/1.1").
     */
    public String getStatusLine();
    
    /**
     * get protocol , link HTTP / 1.1
     * 
     * @return
     */
    public String getProtocolVersion();
    
    /**
     * get respone state code
     * 
     * @return
     */
    public int getStatusCode();
    
    /**
     * get status message
     * 
     * @return
     */
    public String getStatusMessage();
    
    /**
     * get all headers
     * 
     * @return
     */
    public Header[] getAllHeaders();
    
    /**
     * response headers (NOTE: maybe return null)
     */
    public String[] getHeaders(String header);
    
    /**
     * Gets the first header with the given name.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header to get
     * @return the first header or <code>null</code>
     */
    public String getFirstHeader(String name);
    
    /**
     * Gets the last header with the given name.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header to get
     * @return the last header or <code>null</code>
     */
    public String getLastHeader(String name);
    
    /**
     * Gets a header representing all of the header values with the given name. If more that one header with the given
     * name exists the values will be combined with a "," as per RFC 2616.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header(s) to get
     * @return a header string array with a condensed value or <code>null</code> if no headers by the given name are
     *         present
     */
    public String getCondensedHeader(String name);
    
    /**
     * get location, location will null when sync request , because sync request will redirect auto
     * 
     * @return
     */
    public String getLocation();
    
    /**
     * get last modified
     * 
     * @return
     */
    public String getLastModified();
    
    /**
     * get expires
     * 
     * @return
     */
    public String getExpires();
    
    /**
     * get etag
     * 
     * @return
     */
    public String getEtag();
    
    /**
     * get content encoding
     * 
     * @return
     */
    public String getContentEncoding();
    
    /**
     * get content type
     * 
     * @return
     */
    public String getContentType();
    
    /**
     * get content length
     * 
     * @return
     */
    public long getContentLength();
    
    /**
     * get proxy authenticate
     * 
     * @return
     */
    public String getProxyAuthenticate();
    
    /**
     * get www authenticate
     * 
     * @return
     */
    public String getWwwAuthenticate();
    
    /**
     * get transfer encoding
     * 
     * @return
     */
    public String getTransferEncoding();
    
    /**
     * get connection type
     * 
     * @return
     */
    public String getConnectionType();
    
    /**
     * get disposition
     * 
     * @return
     */
    public String getContentDisposition();
    
    /**
     * get accept ranges
     * 
     * @return
     */
    public String getAcceptRanges();
    
    /**
     * get cache control
     * 
     * @return
     */
    public String getCacheControl();
    
    /**
     * get cookies
     * 
     * @return
     */
    public String[] getCookies();
    
    /**
     * get pragma
     * 
     * @return
     */
    public String getPragma();
    
    /**
     * get xpermitted cross domain policies
     * 
     * @return
     */
    public String getXPermittedCrossDomainPolicies();
    
    /**
     * response body
     */
    public InputStream readResponse() throws IOException;
}
