package com.uc.base.net;

/**
 * 网络库管理类，存储构建网络库Client实现工厂, 存储网络库静态功能接口实现工厂，提供全局设置接口如代理等
 */
public class NetworkManager {

    private static volatile NetworkManager sInstance;

    public static String NET_LIB_IMPL_TYPE_LEGACY = "legacy";
    public static String NET_LIB_IMPL_TYPE_UNET = "unet";

    private NetworkManager() {
    }

    private INetClientDelegate mClientDelegate;
    private INetFuncDelegate mFuncDelegate;

    public static NetworkManager getInstance() {
        if (sInstance == null) {
            synchronized (NetworkManager.class) {
                if (sInstance == null) {
                    sInstance = new NetworkManager();
                }
            }
        }
        return sInstance;
    }

    public void init(INetClientDelegate clientDelegate, INetFuncDelegate funcDelegate) {
        mClientDelegate = clientDelegate;
        mFuncDelegate = funcDelegate;
    }

    public INetClientDelegate getNetClientDelegate() {
        return mClientDelegate;
    }

    public INetFuncDelegate getNetFuncDelegate() {
        return mFuncDelegate;
    }

    public void setSystemProxy(String host, int port) {
        mFuncDelegate.setSystemProxy(host, port);
    }

    public String getSystemProxy() {
        return mFuncDelegate.getSystemProxy();
    }

    /**
     * 添加 dns解析结果到dns缓存
     * @param host
     * @param ipsStr  IP列表，用英文","分隔
     * @param ttlSec
     */
    public void addPreResolveDns(String host, String ipsStr, int ttlSec) {
        mFuncDelegate.addPreResolveDns(host, ipsStr, ttlSec);
    }

    public void setArgs(String key, String value) {
        mFuncDelegate.setArgs(key, value);
    }

    public void setArgs(String key, int value) {
        mFuncDelegate.setArgs(key, value);
    }

    public String getNetLibImplType() {
        return mFuncDelegate.getNetImplLibType();
    }

    public boolean isUPaaSEnable() {
        return mFuncDelegate.isUPaaSEnable();
    }

    public void clearDnsCache() {
        mFuncDelegate.clearDnsCache();
    }

    public void clearDnsCache(String host) {
        mFuncDelegate.clearDnsCache(host);
    }

}
