package com.uc.base.net.metrics;

import com.uc.base.net.util.DoubleKeyMap;
import com.uc.base.net.util.Util;

/* package */ class HttpConnectionMetricsBusiness extends HttpConnectionMetricsBase {
    private final DoubleKeyMap<String, HttpConnectionMetricsType, String> mMetricsCache
            = new DoubleKeyMap<String, HttpConnectionMetricsType, String>();

    public HttpConnectionMetricsBusiness() {

    }

    public void increaseMetrics(String tag, HttpConnectionMetricsType metricsType) {
        synchronized (mMetricsCache) {
            long value = Util.StringToLong(mMetricsCache.get(tag, metricsType));
            mMetricsCache.put(tag, metricsType, String.valueOf(value + 1));
        }
    }

    public void addMetrics(String tag, HttpConnectionMetricsType metricsType, String value) {
        synchronized (mMetricsCache) {
            String tmp = mMetricsCache.get(tag, metricsType) + ", "+ value;
            mMetricsCache.put(tag, metricsType, tmp);
        }
    }

    @Override
    public void addMetrics(String tag, HttpConnectionMetricsType metricsType, long value) {
        synchronized (mMetricsCache) {
            long tmp = Util.StringToLong(mMetricsCache.get(tag, metricsType)) + value;
            mMetricsCache.put(tag, metricsType, String.valueOf(tmp));
        }
    }

    public void setMetrics(String tag, HttpConnectionMetricsType metricsType, String value) {
        synchronized (mMetricsCache) {
            mMetricsCache.put(tag, metricsType, value);
        }
    }

    public void resetMetrics(String tag) {
        synchronized (mMetricsCache) {
            mMetricsCache.clear(tag);
        }
    }

    /**
     *
     * @param tag user defined
     * @param metricsType HttpConnectionMetricsType
     * @return metrics by type
     */
    @Override
    public String getMetrics(String tag, HttpConnectionMetricsType metricsType) {
        synchronized (mMetricsCache) {
            return mMetricsCache.get(tag, metricsType);
        }
    }

    /**
     * metrics for sent bytes
     * @return sent bytes
     */
    public long getSentBytesCount(String tag) {
        synchronized (mMetricsCache) {
            return Util.StringToLong(mMetricsCache.get(tag, HttpConnectionMetricsType.METRICS_TYPE_SENT_BYTES_COUNT));
        }
    }

    /**
     * metrics for sent bytes
     * @return sent bytes
     */
    public long getReceivedBytesCount(String tag) {
        synchronized (mMetricsCache) {
            return Util.StringToLong(mMetricsCache.get(tag, HttpConnectionMetricsType.METRICS_TYPE_RECEIVED_BYTES_COUNT));
        }
    }

    /**
     * metrics for request.
     * @return times for request.
     */
    public long getRequestCount(String tag) {
        synchronized (mMetricsCache) {
            return Util.StringToLong(mMetricsCache.get(tag, HttpConnectionMetricsType.METRICS_TYPE_REQUEST_COUNT));
        }
    }

    /**
     * metrics for receive response.
     * @return times for receive response
     */
    public long getResponseCount(String tag) {
        synchronized (mMetricsCache) {
            return Util.StringToLong(mMetricsCache.get(tag, HttpConnectionMetricsType.METRICS_TYPE_RESPONSE_COUNT));
        }
    }
}
