package com.uc.base.net.metrics;

import com.uc.base.net.util.Util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

public class HttpConnectionMetricsConnection extends HttpConnectionMetricsBase implements Cloneable
{
    private String mTag;
    private HashMap<HttpConnectionMetricsType, String> mMetricsCache = new HashMap<HttpConnectionMetricsType, String>();
    private List<HttpConnectionMetricsListener> mListeners = new ArrayList<HttpConnectionMetricsListener>();

    public HttpConnectionMetricsConnection(String tag) {
        mTag = tag;

        int metricsLevel = HttpConnectionMetricsConfig.getMetricsLevel();
        // register listener.
        if ((metricsLevel & HttpConnectionMetricsLevel.METRICS_LEVEL_BUSINESS) > 0) {
            addListener(HttpConnectionMetrics.mMetricsBusiness);
        }

        if ((metricsLevel & HttpConnectionMetricsLevel.METRICS_LEVEL_GLOBAL) > 0) {
            addListener(HttpConnectionMetrics.mMetricsGlobal);
        }
    }

    private void addListener(HttpConnectionMetricsListener listener) {
        if (listener != null) {
            mListeners.add(listener);
        }
    }

    private void removeListener(HttpConnectionMetricsListener listener) {
        if (listener != null) {
            mListeners.remove(listener);
        }
    }

    public void increaseMetrics(HttpConnectionMetricsType metricsType) {
        increaseMetrics(null, metricsType);

        Iterator<HttpConnectionMetricsListener> it =  mListeners.iterator();
        while (it.hasNext()) {
            HttpConnectionMetricsListener listener = it.next();
            if (listener != null) {
                listener.increaseMetrics(mTag, metricsType);
            }
        }
    }

    public void addMetrics(HttpConnectionMetricsType metricsType, long value) {
        addMetrics(null, metricsType, value);

        Iterator<HttpConnectionMetricsListener> it =  mListeners.iterator();
        while (it.hasNext()) {
            HttpConnectionMetricsListener listener = it.next();
            if (listener != null) {
                listener.addMetrics(mTag, metricsType, value);
            }
        }
    }

    public void addMetrics(HttpConnectionMetricsType metricsType, String value) {
        addMetrics(null, metricsType, value);

        Iterator<HttpConnectionMetricsListener> it =  mListeners.iterator();
        while (it.hasNext()) {
            HttpConnectionMetricsListener listener = it.next();
            if (listener != null) {
                listener.addMetrics(mTag, metricsType, value);
            }
        }
    }

    public void setMetrics(HttpConnectionMetricsType metricsType, String value) {
        setMetrics(null, metricsType, value);

        Iterator<HttpConnectionMetricsListener> it =  mListeners.iterator();
        while (it.hasNext()) {
            HttpConnectionMetricsListener listener = it.next();
            if (listener != null) {
                listener.setMetrics(mTag, metricsType, value);
            }
        }
    }

    /**
     * get special metrics
     * @param metricsType HttpConnectionMetricsType
     * @return metrics params
     */
    public String getMetrics(HttpConnectionMetricsType metricsType) {
        return mMetricsCache.get(metricsType);
    }

    @Override
    public String getMetrics(String tag, HttpConnectionMetricsType metricsType) {
        return getMetrics(metricsType);
    }

    /**
     * metrics for sent bytes
     * @return sent bytes
     */
    public long getSentBytesCount(String tag) {
        return Util.StringToLong(mMetricsCache.get(HttpConnectionMetricsType.METRICS_TYPE_SENT_BYTES_COUNT));
    }

    /**
     * metrics for sent bytes
     * @return sent bytes
     */
    public long getReceivedBytesCount(String tag) {
        return Util.StringToLong(mMetricsCache.get(HttpConnectionMetricsType.METRICS_TYPE_RECEIVED_BYTES_COUNT));
    }

    /**
     * metrics for request.
     * @return times for request.
     */
    public long getRequestCount(String tag) {
        return Util.StringToLong(mMetricsCache.get(HttpConnectionMetricsType.METRICS_TYPE_REQUEST_COUNT));
    }

    /**
     * metrics for receive response.
     * @return times for receive response
     */
    public long getResponseCount(String tag) {
        return Util.StringToLong(mMetricsCache.get(HttpConnectionMetricsType.METRICS_TYPE_RESPONSE_COUNT));
    }

    @Override
    public void resetMetrics(String tag) {
        mMetricsCache.clear();
    }

    @Override
    public void increaseMetrics(String tag, HttpConnectionMetricsType metricsType) {
        long tmp = Util.StringToLong(mMetricsCache.get(metricsType)) + 1;
        mMetricsCache.put(metricsType, String.valueOf(tmp));
    }

    @Override
    public void addMetrics(String tag, HttpConnectionMetricsType metricsType, String value) {
        String tmp = mMetricsCache.get(metricsType) + ", " + value;
        mMetricsCache.put(metricsType, String.valueOf(tmp));
    }

    @Override
    public void addMetrics(String tag, HttpConnectionMetricsType metricsType, long value) {
        long tmp = Util.StringToLong(mMetricsCache.get(metricsType)) + value;
        mMetricsCache.put(metricsType, String.valueOf(tmp));
    }

    @Override
    public void setMetrics(String tag, HttpConnectionMetricsType metricsType, String value) {
        mMetricsCache.put(metricsType, value);
    }

    @Override
    public HttpConnectionMetricsConnection clone() {
        HttpConnectionMetricsConnection conn = new HttpConnectionMetricsConnection(mTag);
        conn.mMetricsCache = new HashMap<HttpConnectionMetricsType, String>(mMetricsCache);
        return conn;
    }
}
