/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-24
 *
 * Description : 
 *
 * Creation    : 2014-3-24
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-24, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net.natives;

import com.uc.base.net.adaptor.Headers;
import com.uc.base.net.adaptor.Headers.Header;
import com.uc.annotation.InvokeType;
import com.uc.annotation.Invoker;

/*package*/class NativeHeaders {
    private Headers mHeaders;
    
    public NativeHeaders(Headers handers) {
        mHeaders = handers;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getTransferEncoding() {
        if (mHeaders != null) {
            return mHeaders.getTransferEncoding();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public long getContentLength() {
        if (mHeaders != null) {
            return mHeaders.getContentLength();
        }
        return -1;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getContentType() {
        if (mHeaders != null) {
            return mHeaders.getContentType();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getContentEncoding() {
        if (mHeaders != null) {
            return mHeaders.getContentEncoding();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getLocation() {
        if (mHeaders != null) {
            return mHeaders.getLocation();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getWwwAuthenticate() {
        if (mHeaders != null) {
            return mHeaders.getWwwAuthenticate();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getProxyAuthenticate() {
        if (mHeaders != null) {
            return mHeaders.getProxyAuthenticate();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getContentDisposition() {
        if (mHeaders != null) {
            return mHeaders.getContentDisposition();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getAcceptRanges() {
        if (mHeaders != null) {
            return mHeaders.getAcceptRanges();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getExpires() {
        if (mHeaders != null) {
            return mHeaders.getExpires();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getCacheControl() {
        if (mHeaders != null) {
            return mHeaders.getCacheControl();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getLastModified() {
        if (mHeaders != null) {
            return mHeaders.getLastModified();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getEtag() {
        if (mHeaders != null) {
            return mHeaders.getEtag();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String[] getCookies() {
        if (mHeaders != null) {
            return mHeaders.getCookies();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getPragma() {
        if (mHeaders != null) {
            return mHeaders.getPragma();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getRefresh() {
        if (mHeaders != null) {
            return mHeaders.getRefresh();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getXPermittedCrossDomainPolicies() {
        if (mHeaders != null) {
            return mHeaders.getXPermittedCrossDomainPolicies();
        }
        return null;
        
    }
    
    @Invoker(type = InvokeType.Native)
    public String getConnectionType() {
        if (mHeaders != null) {
            return mHeaders.getConnectionType();
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String[] getHeaders(String header) {
        if (mHeaders != null) {
            return mHeaders.getHeaders(header);
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getFirstHeader(String name) {
        if (mHeaders != null) {
            return mHeaders.getFirstHeader(name);
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getLastHeader(String name) {
        if (mHeaders != null) {
            return mHeaders.getLastHeader(name);
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public String getCondensedHeader(String name) {
        if (mHeaders != null) {
            return mHeaders.getCondensedHeader(name);
        }
        return null;
    }
    
    @Invoker(type = InvokeType.Native)
    public NativeHeader[] getAllHeaders() {
        if (mHeaders == null) {
            return null;
        }
        
        Header[] headers = mHeaders.getAllHeaders();
        if (headers == null || headers.length == 0) {
            return null;
        }
        
        NativeHeader[] nativeHeaders = new NativeHeader[headers.length];
        for (int i = 0; i < headers.length; i++) {
            NativeHeader nativeHeader = new NativeHeader(headers[i]);
            nativeHeaders[i] = nativeHeader;
        }
        return nativeHeaders;
    }
    
    public static class NativeHeader {
        private String name;
        private String value;
        
        @Invoker(type = InvokeType.Native)
        public NativeHeader(NativeHeader header) {
            this.name = header.name;
            this.value = header.value;
        }
        
        public NativeHeader(Header header) {
            name = header.getName();
            value = header.getValue();
        }
        
        public NativeHeader(String name, String value) {
            this.name = name;
            this.value = value;
        }
        
        @Invoker(type = InvokeType.Native)
        public String getName() {
            return name;
        }
        
        public void setName(String name) {
            this.name = name;
        }
        
        @Invoker(type = InvokeType.Native)
        public String getValue() {
            return value;
        }
        
        public void setValue(String value) {
            this.value = value;
        }
        
    }
    
}
