/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-24
 *
 * Description : 
 *
 * Creation    : 2014-3-24
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-24, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net.natives;

import android.os.Looper;

import com.uc.annotation.InvokeType;
import com.uc.annotation.Invoker;
import com.uc.base.net.HttpClientAsync;
import com.uc.base.net.IHttpClientAsync;
import com.uc.base.net.IRequest;

/* package */ class NativeHttpClientAsync {

    private IHttpClientAsync mImpl;
    private NativeHttpEventListener mEventListener;
    
    @Invoker(type = InvokeType.Native)
    public NativeHttpClientAsync(NativeHttpEventListener listener) {
        mImpl = new HttpClientAsync(listener);
        mEventListener = listener;
    }
    
    @Invoker(type = InvokeType.Native)
    public NativeHttpClientAsync(NativeHttpEventListener listener, Looper looper) {
        mImpl = new HttpClientAsync(listener, looper);
        mEventListener = listener;
    }
    
    @Invoker(type = InvokeType.Native)
    public NativeRequest getNativeRequest(String url) {
        NativeRequest request = new NativeRequest(mImpl.getRequest(url));
        return request;
    }
    
    @Invoker(type = InvokeType.Native)
    public void sendRequest(NativeRequest nativeRequest) {
        IRequest request = nativeRequest.getRequest();
        if (request != null) {
            mImpl.sendRequest(request);
        }
    }
    
    @Invoker(type = InvokeType.Native)
    public void sendRequest(NativeRequest nativeRequest, boolean highPrority) {
        IRequest request = nativeRequest.getRequest();
        if (request != null) {
            mImpl.sendRequest(request, highPrority);
        }
    }
    
    @Invoker(type = InvokeType.Native)
    public void setConnectionTimeout(int timeout) {
        mImpl.setConnectionTimeout(timeout);
    }
    
    @Invoker(type = InvokeType.Native)
    public void setSocketTimeout(int timeout) {
        mImpl.setSocketTimeout(timeout);
    }
    
    @Invoker(type = InvokeType.Native)
    public void setAuth(String username, String password) {
        mImpl.setAuth(username, password);
    }
    
    @Invoker(type = InvokeType.Native)
    public void cancel(NativeRequest nativeRequest) {

        if(null != mEventListener){
            mEventListener.cancel();
        }
        
        IRequest request = nativeRequest.getRequest();
        if (request != null) {
            mImpl.cancel(request);
        }
    }

    @Invoker(type = InvokeType.Native)
    public NativeHttpConnectionMetrics getMetrics() {
        return  (new NativeHttpConnectionMetrics(mImpl.getHttpConnectionMetrics()));
    }
}
