package com.uc.base.net.natives;

import com.uc.annotation.InvokeType;
import com.uc.annotation.Invoker;
import com.uc.base.net.IHttpEventListener;
import com.uc.base.net.adaptor.Headers;
import com.uc.base.net.metrics.IHttpConnectionMetrics;

/**
 * this method will construct by native.
 */
/* package */class NativeHttpEventListener implements IHttpEventListener {
    
    private int mNativeListener;
        
    @Invoker(type = InvokeType.Native)
    public NativeHttpEventListener() {
    }
    
    
    /**
     * if call this method to reset the value of member mNativeListener, which is the binded cpp object.
     * after call this method, needn't call any jni mehtod, because mNativeListener is zero.
     */
    public void cancel(){        
        releaseNativeEventListener();
    }
    
    public boolean hasNativeEventListener(){
        return mNativeListener != 0;
    }
    
    @Override
    public void onStatusMessage(String protocol, int statusCode, String statusMessage) {
        
        if( hasNativeEventListener() ) {
            nativeOnStatusMessage(protocol, statusCode, statusMessage);
        }
    }
    
    @Override
    public void onError(int errorId, String errorMsg) {
        
        if( hasNativeEventListener() ) {
            nativeOnError(errorId, errorMsg);
        }
    }
    
    @Override
    public void onHeaderReceived(Headers headers) {
        
        if( hasNativeEventListener() ) {
            nativeOnHeaderReceived(new NativeHeaders(headers));;
        }
    }
    
    @Override
    public void onBodyReceived(byte[] data, int length) {
        //TimeCal.startMonitor(SmoothKeysDef.KEY_RECEIVE_BODY_4);
        
        if( hasNativeEventListener() ) {
            nativeOnBodyReceived(data, length);
        }
        
        //TimeCal.stopMonitor(SmoothKeysDef.KEY_RECEIVE_BODY_4);
    }
    
    @Override
    public void onRequestCancel() {
        if( hasNativeEventListener() ) {
            nativeOnRequestCancel();
        }
    }
    
    @Override
    public boolean onRedirect(String redirectTo) {
        
        if( hasNativeEventListener() ) {
            return nativeOnRedirect(redirectTo);
        }else{
            return true;
        }
        
    }

    @Override
    public void onMetrics(IHttpConnectionMetrics metrics) {
        if( hasNativeEventListener() ) {
            nativeOnMetrics(new NativeHttpConnectionMetrics(metrics));
        }
    }

//    @Override
//    public void onDataDecompressStart(String alg, String dictId) {
//    }
//
//    @Override
//    public void onDataDecompressEnd(String alg, String dictId, int errorId, String errorName) {
//    }

    @Invoker(type = InvokeType.Native)
    private void setNativeEventListener(int listener) {
        mNativeListener = listener;
    }
    
    @Invoker(type = InvokeType.Native)
    private int getNativeEventListner() {
        return mNativeListener;
    }
    
    @Invoker(type = InvokeType.Native)
    private void releaseNativeEventListener() {
        mNativeListener = 0;
    }
    
    // native methods.
    public native void nativeOnStatusMessage(String protocol, int statusCode, String statusMessage);
    
    public native void nativeOnError(int errorId, String errorMsg);
    
    public native void nativeOnHeaderReceived(NativeHeaders headers);
    
    public native void nativeOnBodyReceived(byte[] data, int length);
    
    public native void nativeOnRequestCancel();
    
    public native boolean nativeOnRedirect(String redirectTo);

    public native void nativeOnMetrics(NativeHttpConnectionMetrics metrics);
}
