/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-4-7
 *
 * Description : 
 *
 * Creation    : 2014-4-7
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-4-7, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net.natives;

import com.uc.annotation.InvokeType;
import com.uc.annotation.Invoker;
import com.uc.base.net.IRequest;
import com.uc.base.net.adaptor.Headers.Header;
import com.uc.base.net.natives.NativeHeaders.NativeHeader;

import java.io.InputStream;

@Invoker(type = InvokeType.Native)
/* package */class NativeRequest {
    public IRequest mRequest;
    
    @Invoker(type = InvokeType.Native)
    public NativeRequest(IRequest request) {
        mRequest = request;
    }
    
    public IRequest getRequest() {
        return mRequest;
    }
    
    /**
     * get request ur
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getUrl() {
        if (mRequest != null) {
            mRequest.getUrl();
        }
        return null;
    }
    
    /**
     * set request method, {@link com.uc.base.net.adaptor.Headers#METHOD_GET} {@link com.uc.base.net.adaptor.Headers#METHOD_POST}
     * <p>
     * throw IllegalArgument exception if method is null or invalide
     * 
     * @param method
     */
    @Invoker(type = InvokeType.Native)
    public void setMethod(String method) {
        if (mRequest != null) {
            mRequest.setMethod(method);
        }
    }
    
    /**
     * get current request method
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getMethod() {
        if (mRequest != null) {
            mRequest.getMethod();
        }
        return null;
    }
    
    /**
     * set request content, the input stream and read length
     * 
     * @param bodyProvider
     * @param bodyLength
     */
    @Invoker(type = InvokeType.Native)
    public void setBodyProvider(InputStream bodyProvider, long bodyLength) {
        if (mRequest != null) {
            mRequest.setBodyProvider(bodyProvider, bodyLength);
        }
    }
    
    /**
     * set request content, a file path, only support absolute path and file://path throw IllegalArgumentException if
     * file is't exist
     * 
     * @param pathUrl
     */
    @Invoker(type = InvokeType.Native)
    public void setBodyProvider(String pathUrl) {
        if (mRequest != null) {
            mRequest.setBodyProvider(pathUrl);
        }
    }
    
    /**
     * set request content, byte array
     * 
     * @param bodyProvider
     */
    @Invoker(type = InvokeType.Native)
    public void setBodyProvider(byte[] bodyProvider) {
        if (mRequest != null) {
            mRequest.setBodyProvider(bodyProvider);
        }
    }
    
    /**
     * add request properties
     * 
     * @param key
     * @param value
     */
    @Invoker(type = InvokeType.Native)
    public void addHeader(String key, String value) {
        if (mRequest != null) {
            mRequest.addHeader(key, value);
        }
    }
    
    /**
     * Replaces the first occurence of the header with the same name. If no header with the same name is found the given
     * header is added to the end of the list.
     * 
     * @param header
     *            the new header that should replace the first header with the same name if present in the list.
     */
    @Invoker(type = InvokeType.Native)
    public void updateHeader(String key, String value) {
        if (mRequest != null) {
            mRequest.updateHeader(key, value);
        }
    }
    
    @Invoker(type = InvokeType.Native)
    public NativeHeader[] getAllHeaders() {
        if (mRequest != null) {
            Header[] headers = mRequest.getAllHeaders();
            if (headers != null && headers.length > 0) {
                NativeHeader[] nativeHeaders = new NativeHeader[headers.length];
                for (int i = 0; i < headers.length; i++) {
                    NativeHeader nativeHeader = new NativeHeader(headers[i]);
                    nativeHeaders[i] = nativeHeader;
                }
                return nativeHeaders;
            }
        }
        
        return null;
    }
    
    /**
     * remove request header, it will remove all header with the same name,
     * <p>
     * for example removeHeaders("Set-Cookies"), will remove all cookies
     * 
     * @param header
     */
    @Invoker(type = InvokeType.Native)
    public void removeHeaders(String header) {
        if (mRequest != null) {
            mRequest.removeHeaders(header);
        }
    }
    
    /**
     * Removes the given header.
     * 
     * @param header
     *            the header to remove
     */
    @Invoker(type = InvokeType.Native)
    public void removeHeader(NativeHeader nativeHeader) {
        if (nativeHeader == null) {
            return;
        }
        
        if (mRequest != null) {
            Header header = new Header(nativeHeader.getName(), nativeHeader.getValue());
            mRequest.removeHeader(header);
        }
    }
    
    /**
     * Gets all of the headers with the given name. The returned array maintains the relative order in which the headers
     * were added.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header(s) to get
     * 
     * @return an array of length >= 0
     */
    @Invoker(type = InvokeType.Native)
    NativeHeader[] getHeaders(String header) {
        if (mRequest != null) {
            Header[] headers = mRequest.getHeaders(header);
            if (headers != null && headers.length > 0) {
                NativeHeader[] nativeHeaders = new NativeHeader[headers.length];
                for (int i = 0; i < headers.length; i++) {
                    NativeHeader nativeHeader = new NativeHeader(headers[i]);
                    nativeHeaders[i] = nativeHeader;
                }
                return nativeHeaders;
            }
        }
        
        return null;
    }
    
    /**
     * 
     * @param header
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public boolean containsHeaders(String header) {
        if (mRequest != null) {
            return mRequest.containsHeaders(header);
        }
        return false;
    }
    
    /**
     * set accept encoding
     * 
     * @param string
     */
    @Invoker(type = InvokeType.Native)
    public void setAcceptEncoding(String encoding) {
        if (mRequest != null) {
            mRequest.setAcceptEncoding(encoding);
        }
    }
    
    /**
     * set content type
     * 
     * @param string
     */
    @Invoker(type = InvokeType.Native)
    public void setContentType(String type) {
        if (mRequest != null) {
            mRequest.setContentType(type);
        }
    }
    
}
