/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-4-7
 *
 * Description : 
 *
 * Creation    : 2014-4-7
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-4-7, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net.natives;

import com.uc.annotation.InvokeType;
import com.uc.annotation.Invoker;
import com.uc.base.net.IResponse;
import com.uc.base.net.adaptor.Headers.Header;
import com.uc.base.net.natives.NativeHeaders.NativeHeader;

import java.io.IOException;
import java.io.InputStream;

/* package */class NativeResponse {
    
    private IResponse mResponse;
    
    @Invoker(type = InvokeType.Native)
    public NativeResponse(IResponse response) {
        mResponse = response;
    }
    
    /**
     * remote address
     */
    @Invoker(type = InvokeType.Native)
    public String getRemoteAddress() {
        if (mResponse != null) {
            return mResponse.getRemoteAddress();
        }
        return null;
    }
    
    /**
     * get remote host name
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getRemoteHostName() {
        if (mResponse != null) {
            return mResponse.getRemoteHostName();
        }
        return null;
    }
    
    /**
     * get remote port
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public int getRemotePort() {
        if (mResponse != null) {
            return mResponse.getRemotePort();
        }
        return -1;
    }
    
    /**
     * Sets the response status line (like "HTTP/1.0 200 OK") or request line (like "GET / HTTP/1.1").
     */
    @Invoker(type = InvokeType.Native)
    public String getStatusLine() {
        if (mResponse != null) {
            return mResponse.getStatusLine();
        }
        return null;
    }
    
    /**
     * get protocol , link HTTP / 1.1
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getProtocolVersion() {
        if (mResponse != null) {
            return mResponse.getProtocolVersion();
        }
        return null;
    }
    
    /**
     * get respone state code
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public int getStatusCode() {
        if (mResponse != null) {
            return mResponse.getStatusCode();
        }
        return -1;
    }
    
    /**
     * get status message
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getStatusMessage() {
        if (mResponse != null) {
            return mResponse.getStatusMessage();
        }
        return null;
    }
    
    /**
     * get all headers
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public NativeHeader[] getAllHeaders() {
        if (mResponse != null) {
            Header[] headers = mResponse.getAllHeaders();
            if (headers == null || headers.length == 0) {
                return null;
            }
            
            NativeHeader[] nativeHeaders = new NativeHeader[headers.length];
            for (int i = 0; i < headers.length; i++) {
                NativeHeader nativeHeader = new NativeHeader(headers[i]);
                nativeHeaders[i] = nativeHeader;
            }
            return nativeHeaders;
        }
        return null;
    }
    
    /**
     * response headers (NOTE: maybe return null)
     */
    @Invoker(type = InvokeType.Native)
    public String[] getHeaders(String header) {
        if (mResponse != null) {
            return mResponse.getHeaders(header);
        }
        return null;
    }
    
    /**
     * Gets the first header with the given name.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header to get
     * @return the first header or <code>null</code>
     */
    @Invoker(type = InvokeType.Native)
    public String getFirstHeader(String name) {
        if (mResponse != null) {
            return mResponse.getFirstHeader(name);
        }
        return null;
    }
    
    /**
     * Gets the last header with the given name.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header to get
     * @return the last header or <code>null</code>
     */
    @Invoker(type = InvokeType.Native)
    public String getLastHeader(String name) {
        if (mResponse != null) {
            return mResponse.getLastHeader(name);
        }
        return null;
    }
    
    /**
     * Gets a header representing all of the header values with the given name. If more that one header with the given
     * name exists the values will be combined with a "," as per RFC 2616.
     * 
     * <p>
     * Header name comparison is case insensitive.
     * 
     * @param name
     *            the name of the header(s) to get
     * @return a header string array with a condensed value or <code>null</code> if no headers by the given name are
     *         present
     */
    @Invoker(type = InvokeType.Native)
    public String getCondensedHeader(String name) {
        if (mResponse != null) {
            return mResponse.getCondensedHeader(name);
        }
        return null;
    }
    
    /**
     * get location, location will null when sync request , because sync request will redirect auto
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getLocation() {
        if (mResponse != null) {
            return mResponse.getLocation();
        }
        return null;
    }
    
    /**
     * get last modified
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getLastModified() {
        if (mResponse != null) {
            return mResponse.getLastModified();
        }
        return null;
    }
    
    /**
     * get expires
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getExpires() {
        if (mResponse != null) {
            return mResponse.getExpires();
        }
        return null;
    }
    
    /**
     * get etag
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getEtag() {
        if (mResponse != null) {
            return mResponse.getEtag();
        }
        return null;
    }
    
    /**
     * get content encoding
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getContentEncoding() {
        if (mResponse != null) {
            return mResponse.getContentEncoding();
        }
        return null;
    }
    
    /**
     * get content type
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getContentType() {
        if (mResponse != null) {
            return mResponse.getContentType();
        }
        return null;
    }
    
    /**
     * get content length
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public long getContentLength() {
        if (mResponse != null) {
            return mResponse.getContentLength();
        }
        return -1;
    }
    
    /**
     * get proxy authenticate
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getProxyAuthenticate() {
        if (mResponse != null) {
            return mResponse.getProxyAuthenticate();
        }
        return null;
    }
    
    /**
     * get www authenticate
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getWwwAuthenticate() {
        if (mResponse != null) {
            return mResponse.getWwwAuthenticate();
        }
        return null;
    }
    
    /**
     * get transfer encoding
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getTransferEncoding() {
        if (mResponse != null) {
            return mResponse.getTransferEncoding();
        }
        return null;
    }
    
    /**
     * get connection type
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getConnectionType() {
        if (mResponse != null) {
            return mResponse.getConnectionType();
        }
        return null;
    }
    
    /**
     * get disposition
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getContentDisposition() {
        if (mResponse != null) {
            return mResponse.getContentDisposition();
        }
        return null;
    }
    
    /**
     * get accept ranges
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getAcceptRanges() {
        if (mResponse != null) {
            return mResponse.getAcceptRanges();
        }
        return null;
    }
    
    /**
     * get cache control
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getCacheControl() {
        if (mResponse != null) {
            return mResponse.getCacheControl();
        }
        return null;
    }
    
    /**
     * get cookies
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String[] getCookies() {
        if (mResponse != null) {
            return mResponse.getCookies();
        }
        return null;
    }
    
    /**
     * get pragma
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getPragma() {
        if (mResponse != null) {
            return mResponse.getPragma();
        }
        return null;
    }
    
    /**
     * get xpermitted cross domain policies
     * 
     * @return
     */
    @Invoker(type = InvokeType.Native)
    public String getXPermittedCrossDomainPolicies() {
        if (mResponse != null) {
            return mResponse.getXPermittedCrossDomainPolicies();
        }
        return null;
    }
    
    /**
     * response body
     */
    @Invoker(type = InvokeType.Native)
    public InputStream readResponse() throws IOException {
        if (mResponse != null) {
            return mResponse.readResponse();
        }
        return null;
    }
    
}
