/**
 * ****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-4
 * <p>
 * Description :
 * <p>
 * Creation    : 2014-3-4
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-4, linhz, Create the file
 * ****************************************************************************
 */
package com.uc.base.net.unet;

//import com.uc.base.net.adaptor.HttpHost;
//import com.uc.base.net.adaptor.Request;

import android.text.TextUtils;

import com.alibaba.mbg.unet.IProxyDelegate;
import com.alibaba.mbg.unet.Response;
import com.alibaba.mbg.unet.UnetManager;

import java.util.List;
import java.util.Map;

import com.uc.base.net.IRequest;
import com.uc.base.net.adaptor.Headers;
import com.uc.base.net.unet.adaptor.UnetRequestAdaptor;
import com.uc.base.net.metrics.HttpConnectionMetrics;
import com.uc.base.net.metrics.HttpConnectionMetricsConnection;
import com.uc.base.net.metrics.HttpConnectionMetricsType;
import com.uc.base.net.metrics.IHttpConnectionMetrics;
import com.uc.base.net.unet.util.HttpUtil;
import com.uc.base.net.unet.util.IPEndPoint;
import com.uc.base.net.unet.util.NetLog;
import com.uc.base.net.unet.util.ProxyMap;
import com.uc.base.net.util.UrlParser;

/* package */abstract class HttpClientBase {
    /**
     * 0 表示使用uNet默认超时时间，（当前是40S）。
     */
    protected int mConnectTimeOut = 0;
    /**
     * Allow a TCP connection 60 idle seconds before erroring out
     */
    protected int mSocketTimeOut = 60 * 1000;

    /**
     * auth username
     */
    protected String mUsername;
    /**
     * auth password
     */
    protected String mPassword;

    /**
     * proxy host
     */
//    protected HttpHost mProxyHost;

    /**
     * connected host
     */
//    protected HttpHost mHttpHost;
    /**
     * HTTP connection metrics tag
     */
    private String mMetricsTag;

    protected HttpConnectionMetricsConnection mMetricsConnection = null; // just for this request
    protected HttpConnectionMetrics mMetrics = null; // maybe for one business or global.

    UnetManager mShellManager = null;

    private String mProxyHost;
    private int mProxyPort;

    /**
     * set connection timeout , unit : ms
     *
     * @param timeout
     */
    public void setConnectionTimeout(int timeout) {
        mConnectTimeOut = timeout;
    }

    /**
     * set socket timeout , unit : ms
     *
     * @param timeout
     */
    public void setSocketTimeout(int timeout) {
        mSocketTimeOut = timeout;
    }

    /**
     * user name and password will be use when web server need an authentication
     *
     * @param username
     * @param password
     */
    public void setAuth(String username, String password) {
        mUsername = username;
        mPassword = password;
    }

    /**
     * set the proxy ip
     *
     * @param url  the proxy url, for example http://www.ucweb.com
     * @param port the proxy port, for example 8080
     */
    public void setProxy(String url, int port) {
        mProxyHost = url;
        mProxyPort = port;
    }

    public void setFollowProxy(boolean follow) {
        if (!follow) {
            mProxyHost = null;
            mProxyPort = 0;
        }
    }

    /**
     * get the {@link com.uc.base.net.IRequest}, and setup the request properties before start a network request
     *
     * @return {@link com.uc.base.net.IRequest}
     */
    public IRequest getRequest(String url) throws IllegalArgumentException {
        NetLog.e("shellunet", "HttpClientBase getRequest url:" + url);

        if (!url.startsWith("http") && !url.startsWith("https")) {
            // 如果URL不以HTTP或者HTTPS开头，那么交给UrlParser处理Schema，
            // 如果是特殊Schema，那么不添加默认头，如果没有Schema，那么添加默认HTTP头
            try {
                UrlParser parser = new UrlParser(url);
                url = parser.toString();
            } catch (Exception e) {
                e.printStackTrace();
                // 如果UrlParser解析url出错，那么忽略，仍然交给UNET尝试
            }
        }

        mShellManager = UNetContext.getUNetManager();

        // check set proxy
        if (!TextUtils.isEmpty(mProxyHost)) {
            ProxyMap.addProxyInfo(url, mProxyHost + ":" + mProxyPort);
        } else {
            ProxyMap.removeProxyInfo(url);
        }

        return new UnetRequestAdaptor(mShellManager, url);
    }

    /**
     * set HTTP connection metrics
     *
     * @param tag indicate which metrics belongs to.
     */
    public void setMetricsTAG(String tag) {
        mMetricsTag = tag;
    }

    public String getMetricsTAG() {
        return mMetricsTag;
    }

    public IHttpConnectionMetrics getHttpConnectionMetrics() {
        return mMetrics;
    }

    protected void CreateMetrics() {
        //NOTICE:tag should be set after HttpClientXXX create and before request send out.
        mMetricsConnection = new HttpConnectionMetricsConnection(mMetricsTag);
        mMetrics = new HttpConnectionMetrics(mMetricsConnection);
    }

    protected void FormatMetrics(Response iResponse) {
        if (null == iResponse) {
            NetLog.e("shellunet", "HttpClientBase FormatMetrics iResponse null. ");
            return;
        }
        NetLog.e("shellunet", "HttpClientBase FormatMetrics ");
        //do metrics.
        //TODO: full fill this array and share this info with unet. maybe LoadTimeInfo?
        String[] metricinfo = iResponse.getMetricInfoArray();
        if(metricinfo.length >= 11){
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_SENT_BYTES_COUNT, metricinfo[5]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_RECEIVED_BYTES_COUNT, metricinfo[6]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_REQUEST_COUNT, "1");
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_RESPONSE_COUNT, "1");
            IPEndPoint ip = IPEndPoint.parseFromString(metricinfo[10]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_REMOTE_ADDRESS, ip.ip);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_REMOTE_PORT, String.valueOf(ip.port));
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_DNS_PARSE_TIME, metricinfo[0]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_CONNECTION_TIME, metricinfo[1]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_RTT_TIME, metricinfo[4]);

            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_CONNECT_COUNT, metricinfo[7]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_USERVER_STATUS, metricinfo[8]);
            mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_USERVER_ERRORCODE, metricinfo[9]);

            if (metricinfo.length >= 12) {
                mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_LINKUP_URL, metricinfo[11]);
            }

            if (metricinfo.length >= 13) {
                mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_USERVER_MASTER_URL, metricinfo[12]);
            }

        }
        mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_RECEIVED_BYTES_COUNT, String.valueOf(iResponse.getReceivedBytesCount()));

        String contentEncoding = HttpUtil.getHeaderFromResponse(iResponse, Headers.CONTENT_ENCODING);
        String decompressType = "none";
        if (Headers.ENCODING_GZIP.equalsIgnoreCase(contentEncoding)) {
            decompressType = "gzip";
        } else if (Headers.ENCODING_ZSTD.equalsIgnoreCase(contentEncoding)) {
            String zstdDictid = HttpUtil.getHeaderFromResponse(iResponse, Headers.ZSTD_DICTID);
            if (!TextUtils.isEmpty(zstdDictid)) {
                mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_ZSTD_DICT_ID, zstdDictid);
                decompressType = "zstd_dict";
            } else {
                decompressType = "zstd";
            }
        }
        mMetricsConnection.setMetrics(HttpConnectionMetricsType.METRICS_TYPE_DECOMPRESS_ALG, decompressType);
    }

    protected void NotifyRequestComplete(String url, int statusCode, String statusText, Map<String, List<String>> header) {
        if (mShellManager == null) return;
        IProxyDelegate delegate = mShellManager.getProxyDelegate();
        if (delegate == null) return;

        delegate.onRequestComplete(url, statusCode, statusText, header);
    }
}
