/**
 * ****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-4
 * <p>
 * Description :
 * <p>
 * Creation    : 2014-3-4
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-4, linhz, Create the file
 * ****************************************************************************
 */
package com.uc.base.net.unet;

import com.alibaba.mbg.unet.RequestException;
import com.alibaba.mbg.unet.Response;

import com.uc.base.net.IHttpClientSync;
import com.uc.base.net.IResponse;
import com.uc.base.net.unet.adaptor.UnetResponseAdaptor;
import com.uc.base.net.unet.util.NetLog;

public class HttpClientSync extends HttpClientBase implements IHttpClientSync {
    private com.uc.base.net.unet.adaptor.UnetRequestAdaptor mRequestAdaptor = null;
    private Response mResponse = null;
    private boolean mFollowRedirects = true;
    // default: no timeout just waiting net event , using unet timeout.
    // mRequestTimeOut (int) request timeout,only for sync request.
    private static final int DEFAULT_SYNC_REQUEST_TIMEOUT = 60 * 1000;
    private int mRequestTimeOut = -1;

    private RequestException mRequestException = null;

    public HttpClientSync() {

    }

    public IResponse sendRequest(com.uc.base.net.IRequest request) {
        CreateMetrics();
        if (request instanceof com.uc.base.net.unet.adaptor.UnetRequestAdaptor) {
            mRequestAdaptor = (com.uc.base.net.unet.adaptor.UnetRequestAdaptor) request;
            BeforeSyncRequestStart();

            try {
                NetLog.e("shellunet", "shellunet UnetSyncRequest request.start mRequestAdaptor:" + mRequestAdaptor);
                mResponse = mRequestAdaptor.start();
                FormatMetrics(mResponse);
                NetLog.e("shellunet", "#unet HttpClientSync end. url:" + mResponse.getUrl() + " mRequestAdaptor:" + mRequestAdaptor);

                NetLog.e("shellunet", "shellunet UnetSyncRequest request.start end. response:" + mResponse.getHttpStatusText() + " rc:" + mResponse.getHttpStatusCode() + mResponse.getAllHeaders());

                if(mResponse != null){
                    NotifyRequestComplete(mResponse.getUrl(), mResponse.getHttpStatusCode(), mResponse.getHttpStatusText(), mResponse.getAllHeaders());
                } else {
                    NotifyRequestComplete(mRequestAdaptor.getUrl(), errorCode(), null, null);
                }
                return new UnetResponseAdaptor(mResponse);
            } catch (RequestException e) {
                mRequestException = e;
                NetLog.e("shellunet", "shellunet UnetSyncRequest request.start end. RequestException:" + e);

            }
        }
        return null;
    }


    public void close() {
        NetLog.e("shellunet", "shellunet UnetSyncRequest close");
        if (mRequestAdaptor == null) {
            return;
        }
        mRequestAdaptor.cancel();
    }

    public int errorCode() {
        return (mRequestException != null) ? mRequestException.getErrorCode() : 0;
    }

    public void followRedirects(boolean auto) {
        mFollowRedirects = auto;
    }

    @Override
    public void recycle(final int keepTime) {
        // no impl in unet
    }

    @Override
    public void setRequestTimeout(int ms) {
        mRequestTimeOut = ms;
    }

    private void BeforeSyncRequestStart() {
        NetLog.e("shellunet", "shellunet httpClientSync BeforeSyncRequestStart");

        mRequestAdaptor.setAutoFollowRedirect(mFollowRedirects);
        if(mRequestTimeOut > 0){
            mRequestAdaptor.useSyncRequestTimeout(mRequestTimeOut);
        }
        if (mConnectTimeOut > 0) {
            mRequestAdaptor.setConnectTimeout(mConnectTimeOut);
        }
    }

    @Override
    public void setProxy(String url, int port) {
        super.setProxy(url, port);
    }

    @Override
    public void setFollowProxy(boolean follow) {
        super.setFollowProxy(follow);
    }
}
