package com.uc.base.net.unet;


import com.alibaba.mbg.unet.IProxyDelegate;
import com.alibaba.mbg.unet.UnetManager;
import com.alibaba.mbg.upaas.UpaasManager;

import android.content.Context;
import android.text.TextUtils;

import com.uc.base.net.unet.util.ProxyMap;
import com.uc.base.net.unet.util.SecurityGuardWrapperImpl;

import java.util.List;
import java.util.Map;

public class UNetContext{
    private static UnetManager sUnetManager;
    private static UnetManager.Builder sBuilder;
    private static boolean sFollowSystemProxy = true;
    private static String sProxyHost;
    private static int sProxyPort;

    public static void initializeUNetContext(final Context ctx) {
        initializeUNetContext(ctx, null);
    }

    /**
     * 初始化UNet
     * @param ctx
     * @param dpSoPath 动态部署SoPath, 必须是动态部署的SoPath
     */
    public static void initializeUNetContext(final Context ctx, String dpSoPath) {
        sBuilder = new UnetManager.Builder();
        if (!TextUtils.isEmpty(dpSoPath)) {
            sBuilder.setDynamicDeploySoPath(dpSoPath);
        }
        // builder.setContext()是必须的
        sBuilder.setContext(ctx);
    }

    /**
     * 为UNET注入默认的代理实现，可以针对请求设置代理域名同时也启用全局代理能力（通过setSystemProxy实现）
     */
    public static void configDefaultProxy() {
        getUNetManager().setProxyInfoDelegate(new IProxyDelegate() {
            @Override
            public ProxyInfo generateProxyInfo(String s) {
                // ProxyMap 由请求填充
                String proxyUrl = ProxyMap.getProxyInfo(s);

                if (TextUtils.isEmpty(proxyUrl) && sFollowSystemProxy) {
                    if (!TextUtils.isEmpty(sProxyHost) && sProxyPort > 0) {
                        proxyUrl = sProxyHost + ":" + sProxyPort;
                    }
                }

                if (TextUtils.isEmpty(proxyUrl)) {
                    return null;
                }
                ProxyInfo info = new ProxyInfo();
                info.proxyServer = proxyUrl;
                return info;
            }

            @Override
            public void onRequestComplete(String s, int i, String s1, Map<String, List<String>> map) {

            }
        });
    }

    // 是否follow系统代理
    public static void setFollowSystemProxy(boolean followSystemProxy) {
        sFollowSystemProxy = followSystemProxy;
    }

    public static void addSoSearchPath(String path) { sBuilder.addSoSearchPath(path); }

    public static void setSystemProxy(String host, int port) {
        if (TextUtils.isEmpty(host)) {
            return;
        }
        sProxyHost = host;
        sProxyPort = port;
    }

    public static UnetManager getUNetManager() {
        // builder.setCacheDirPath()是可选的，不设置则保存cache到默认路径
        // 其它进程中的UnetManager实例应该设置到不同路径，避免读写文件竞争以致文件损坏
        // builder.setCacheDirPath("/sdcard/dir_path");
        synchronized (UNetContext.class){
            if( null == sUnetManager){
                sUnetManager = sBuilder.build();
            }
        }
        return sUnetManager;
    }

    public static void initializeUPaasManager(Context context, String appName, String ds, String dsType, String appKey, String authCode){
        SecurityGuardWrapperImpl wrapper = new SecurityGuardWrapperImpl(context, appKey, authCode);
        UpaasManager.init(getUNetManager(),
            appName,  // eg: uc_browser
            ds,  // ds maybe udtid
            dsType,  // dsType，还可以是"uuid"
            wrapper);
    }
}