package com.uc.base.net.unet.adaptor;



import com.alibaba.mbg.unet.RequestException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Executor;

import android.text.TextUtils;

import com.alibaba.mbg.unet.Request;
import com.alibaba.mbg.unet.Response;

import com.alibaba.mbg.unet.UnetManager;

import com.uc.base.net.IRequest;
import com.uc.base.net.adaptor.Headers;
import com.uc.base.net.unet.util.NetLog;

/**
 * Created by will on 17-6-2.
 */

public class UnetRequestAdaptor implements IRequest {

    private Request mUnetRequest ;
    private String mInitURL = null;
    private HashMap<String, Headers.Header> mHeaders = new HashMap<String, Headers.Header>();
    private boolean mZstdSupportFlag;
    private boolean mIsUseBackupIp;

    // see load_flags_list.h in chromium net.
    private final int LOAD_FLAG_EXT_IGNORE_SSL_ERROR = 1<<20;
    private final int LOAD_FLAG_EXT_DISABLE_HTTP2 = 1<<21;

    public UnetRequestAdaptor(UnetManager shellManager, String url){
        NetLog.e("shellunet", "UnetRequestAdaptor UnetRequestAdaptor url:" + url);
        //TODO:should access from UCCore to get manager.
//        com.uc.webview.export.extension.net.IManager shellManager = (com.uc.webview.export.extension.net.IManager) com.uc.net.ManagerAdapter.getPlatformNetManager(context);
        //TODO: should check shellManager and throw exception.
        mUnetRequest = shellManager.createRequest(url);
        mInitURL = url;
        NetLog.e("shellunet","UnetRequestAdaptor UnetRequestAdaptor createRequest :"+mUnetRequest);

        NetLog.e("shellunet","#unet UnetRequestAdaptor createRequest url: "+url);

    }

    public void start(Request.Callback callback){
        NetLog.e("shellunet","UnetRequestAdaptor start Request :"+mUnetRequest);
        fillHeadersWithZstdSupport();
        mUnetRequest.disableCache();
        mUnetRequest.start(callback);
    }

    public void start(Executor executor, Request.Callback callback){
        fillHeadersWithZstdSupport();
        mUnetRequest.disableCache();
        mUnetRequest.start(executor,callback);
    }

    public Response start() throws RequestException {
        fillHeadersWithZstdSupport();
        return mUnetRequest.start();
    }

    public void cancel(){
        mUnetRequest.cancel();
    }
    /**
     * get request url
     *
     * @return
     */
    @Override
    public String getUrl() {
        NetLog.e("shellunet","UnetRequestAdaptor getUrl :");
        return mInitURL;
    }

    /**
     * set request method, {@link com.uc.base.net.adaptor.Headers#METHOD_GET} {@link com.uc.base.net.adaptor.Headers#METHOD_POST}
     * <p>
     * throw IllegalArgument exception if method is null or invalide
     *
     * @param method
     */
    @Override
    public void setMethod(String method) {
        NetLog.e("shellunet","UnetRequestAdaptor setMethod :"+method);

        mUnetRequest.setHttpMethod(method);
    }

    /**
     * get current request method
     *
     * @return
     */
    @Override
    public String getMethod() {
        return mUnetRequest.getHttpMethod();
    }

    /**
     * add request properties
     *
     * @param key
     * @param value
     */
    @Override
    public void addHeader(String key, String value) {
        NetLog.e("shellunet","UnetRequestAdaptor addHeader key :"+key+ " value:"+value);
        if(TextUtils.isEmpty(key) || TextUtils.isEmpty(value)){
            return;
        }

        // FIXME: 信息流临时处理
        if (key.equals("iflow-gz2")) {
            // unet 不支持gz2
            return;
        }
        //

        mHeaders.put(key, new Headers.Header(key, value));
    }

    /**
     * Replaces the first occurence of the header with the same name. If no header with the same name is found the given
     * header is added to the end of the list.
     *
     * @param key
     * @param value
     */
    @Override
    public void updateHeader(String key, String value) {
        NetLog.e("shellunet","UnetRequestAdaptor updateHeader no impl in unet. key :"+key+ " value:"+value);

        mHeaders.put(key, new Headers.Header(key, value));
    }

    /**
     * Replaces the first occurence of the header with the same name. If no header with the same name is found the given
     * header is added to the end of the list.
     *
     * @param header the new header that should replace the first header with the same name if present in the list.
     */
    @Override
    public void updateHeader(Headers.Header header) {
        NetLog.e("shellunet","UnetRequestAdaptor updateHeader no impl in unet. header :"+header);

        if (header == null || TextUtils.isEmpty(header.getName())) {
            return;
        }
        mHeaders.put(header.getName(), header);
    }

    /**
     * add headers
     *
     * @param headers
     */
    @Override
    public void addHeaders(ArrayList<Headers.Header> headers) {
        NetLog.e("shellunet","UnetRequestAdaptor addHeaders");

        if (headers == null || headers.size() == 0) {
            return;
        }

        for (Headers.Header header : headers) {
            String key = header.getName();
            if (TextUtils.isEmpty(key)) {
                continue;
            }
            mHeaders.put(key, header);
        }
    }


    /**
     * set request content, the input stream and read length
     *
     * @param bodyProvider
     * @param bodyLength
     */
    @Override
    public void setBodyProvider(InputStream bodyProvider, long bodyLength) {
        NetLog.e("shellunet","UnetRequestAdaptor setBodyProvider by InputStream len:" + bodyLength);
        // unet不支持该接口，这里做临时处理，如果bodyLength小于10MB，转换为byte数组
        // 目前没发现浏览器在调用该接口，保险起见先这样处理，后续移除该接口
        if (bodyLength > 0 && bodyLength <= 10 * 1024 * 1024) {
            byte[] buff = new byte[(int)bodyLength];
            try {
                bodyProvider.read(buff);
                mUnetRequest.setUploadDataProvider(buff);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * set request content, a file path, only support absolute path and file://path throw IllegalArgumentException if
     * file is't exist
     *
     * @param pathUrl
     */
    @Override
    public void setBodyProvider(String pathUrl) {
        NetLog.e("shellunet","UnetRequestAdaptor setBodyProvider pathUrl :"+pathUrl);

        mUnetRequest.setUploadDataProvider(pathUrl);
    }

    /**
     * set request content, byte array
     *
     * @param bodyProvider
     */
    @Override
    public void setBodyProvider(byte[] bodyProvider) {
        NetLog.e("shellunet","UnetRequestAdaptor setBodyProvider :");

        mUnetRequest.setUploadDataProvider(bodyProvider);
    }

    /**
     * get all of headers
     *
     * @return
     */
    @Override
    public Headers.Header[] getAllHeaders() {
        NetLog.e("shellunet","UnetRequestAdaptor getAllHeaders");

        Headers.Header[] headers = new Headers.Header[mHeaders.size()];
        headers = mHeaders.values().toArray(headers);
        return headers;
    }

    /**
     * remove request header, it will remove all header with the same name,
     * <p>
     * for example removeHeaders("Set-Cookies"), will remove all cookies
     *
     * @param header
     */
    @Override
    public void removeHeaders(String header) {
        NetLog.e("shellunet","UnetRequestAdaptor removeHeaders :" + header);

        //throw new UnsupportedOperationException("no impl in unet. ");
        mHeaders.remove(header);
    }

    /**
     * Removes the given header.
     *
     * @param header the header to remove
     */
    @Override
    public void removeHeader(Headers.Header header) {
        NetLog.e("shellunet","UnetRequestAdaptor removeHeader:" + header);

        for (Map.Entry<String, Headers.Header> entry : mHeaders.entrySet()) {
            if (entry.getValue() == header) {
                mHeaders.remove(entry.getKey());
                break;
            }
        }
    }

    /**
     * Gets all of the headers with the given name. The returned array maintains the relative order in which the headers
     * were added.
     * <p>
     * <p>
     * Header name comparison is case insensitive.
     *
     * @param header@return an array of length >= 0
     */
    @Override
    public Headers.Header[] getHeaders(String header) {
        NetLog.e("shellunet","UnetRequestAdaptor getHeaders :" + header);

        Headers.Header h = mHeaders.get(header);
        if (h != null) {
            return new Headers.Header[] {h};
        }
        return null;
    }

    /**
     * @param header
     * @return
     */
    @Override
    public boolean containsHeaders(String header) {
        boolean contains = mHeaders.containsKey(header);
        NetLog.e("shellunet","UnetRequestAdaptor containsHeaders header:" + header + " contains:" + contains);
        return contains;
    }

    /**
     * set accept encoding
     *
     * @param string
     */
    @Override
    public void setAcceptEncoding(String string) {
        NetLog.e("shellunet","UnetRequestAdaptor setAcceptEncoding :"+string + " url:" + mUnetRequest.getURL());

        mHeaders.put(Headers.ACCEPT_ENCODING, new Headers.Header(Headers.ACCEPT_ENCODING, string));
    }

    /**
     * set content type
     *
     * @param string
     */
    @Override
    public void setContentType(String string) {
        NetLog.e("shellunet","UnetRequestAdaptor setContentType :"+string);

        mHeaders.put(Headers.CONTENT_TYPE, new Headers.Header(Headers.CONTENT_TYPE, string));
    }

    // 以下函数在unet中有处理但是没相关接口
    @Override
    public void recordUseBackupDnsIp() {

    }

    @Override
    public boolean isUseBackupDnsIp() {
        return mIsUseBackupIp;
    }

    public void setUseBackupIp(boolean useBackupIp) {
        mIsUseBackupIp = useBackupIp;
    }

    @Override
    public boolean setZstdSupport(final boolean isSupport) {
        NetLog.e("shellunet","setZstdSupport:" + isSupport + " url:" + mUnetRequest.getURL());
        mZstdSupportFlag = isSupport;
        return true;
    }

    @Override
    public boolean isZstdSupport() {
        return mZstdSupportFlag;
    }


    public void setAutoFollowRedirect(boolean auto){
        mUnetRequest.setAutoFollowRedirect(auto);
    }

    public void useSyncRequestTimeout(int ms){
        NetLog.e("shellunet", "shellunet UnetRequestAdaptor useSyncRequestTimeout setRequestTimeout");
        mUnetRequest.setRequestTimeout(ms);
    }

    public void setConnectTimeout(int timeoutMs) {
        mUnetRequest.setConnectTimeout(timeoutMs);
    }

    public void fillHeadersWithZstdSupport() {
        // FIXME: 信息流临时处理, InfoFlowRequestHandler类会先设置AcceptEncoding为gzip，在设置支持zstd
        // 这里和unet2底层库有个冲突，uent2托管zstd请求，但是前提是AcceptEncoding没有设置过。
        // 所有这里将AcceptEncoding移除, 以交给unet2托管zstd
        // 该场景也适用于通用逻辑，一旦设置了支持zstd，那么必须移除用户设置的Accept-Encoding，否则底层不处理
        List<Headers.Header> headers = new LinkedList<Headers.Header>();
        for (Map.Entry<String, Headers.Header> entry : mHeaders.entrySet()) {
            if (mZstdSupportFlag && Headers.ACCEPT_ENCODING.equalsIgnoreCase(entry.getKey())) {
                continue;
            }
            Headers.Header header = entry.getValue();
            String value = header == null ? "" : header.getValue();
            NetLog.e("shellunet", "fillHeader:" + entry.getKey() + " - " + value + " url:" + mUnetRequest.getURL());
            mUnetRequest.addHeader(entry.getKey(), value);
        }
    }

    @Override
    public void ignoreSSLErrorIfOccurs(){
        NetLog.e("shellunet", "shellunet UnetRequestAdaptor ignoreSSLErrorIfOccurs");
        mUnetRequest.setLoadFlagExt(LOAD_FLAG_EXT_IGNORE_SSL_ERROR);
    }

    @Override
    public void disableHttp2() {
        NetLog.e("shellunet", "shellunet UnetRequestAdaptor disableHttp2");
        mUnetRequest.setLoadFlagExt(LOAD_FLAG_EXT_DISABLE_HTTP2);
    }

    @Override
    public void setCookieEnable(boolean enable) {
        mUnetRequest.setCookieEnable(enable);
    }

}

