package com.uc.base.net.unet.adaptor;


import com.uc.base.net.IResponse;
import com.uc.base.net.adaptor.Headers;
import com.uc.base.net.unet.util.HttpUtil;
import com.uc.base.net.unet.util.NetLog;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.alibaba.mbg.unet.Response;

public class UnetResponseAdaptor implements IResponse {
    private Response mUnetResponse;

    public UnetResponseAdaptor(Response resp) {
        mUnetResponse = resp;
    }

    /**
     * remote address
     */
    @Override
    public String getRemoteAddress() {
        return null;
    }

    /**
     * get remote host name
     *
     * @return
     */
    @Override
    public String getRemoteHostName() {
        return null;
    }

    /**
     * get remote port
     *
     * @return
     */
    @Override
    public int getRemotePort() {
        return 0;
    }

    /**
     * Sets the response status line (like "HTTP/1.0 200 OK") or request line (like "GET / HTTP/1.1").
     */
    @Override
    public String getStatusLine() {
        return mUnetResponse.getHttpStatusText();
    }

    /**
     * get protocol , link HTTP / 1.1
     *
     * @return
     */
    @Override
    public String getProtocolVersion() {
        return HttpUtil.getProtocolVersion(mUnetResponse);
    }

    /**
     * get respone state code
     *
     * @return
     */
    @Override
    public int getStatusCode() {
        return mUnetResponse.getHttpStatusCode();
    }

    /**
     * get status message
     *
     * @return
     */
    @Override
    public String getStatusMessage() {
        return mUnetResponse.getHttpStatusText();
    }

    /**
     * get all headers
     *
     * @return
     */
    @Override
    public Headers.Header[] getAllHeaders() {
        Headers shell_headers = new Headers();
        List<Map.Entry<String, String>> headers = mUnetResponse.getAllHeadersAsList();
        for (Map.Entry<String, String> entry : headers) {
            //TODO: value should be a list.means one key should match to a list value,like set-cookies.
            //TODO: make sure getAllHeaders return normal.
            Headers.Header header = new Headers.Header(entry.getKey(), entry.getValue());
            shell_headers.addHeader(header);
        }

        NetLog.e("unet", "UnetResponseAdaptor getAllHeaders:" + shell_headers.toString());
        return shell_headers.getAllHeaders();
    }

    /**
     * response headers (NOTE: maybe return null)
     *
     * @param header
     */
    @Override
    public String[] getHeaders(String header) {
        if (header == null) {
            return null;
        }

        ArrayList<String> res = new ArrayList<String>(4);
        List<Map.Entry<String, String>> headers = mUnetResponse.getAllHeadersAsList();
        for (Map.Entry<String, String> entry : headers) {
            if (header.equalsIgnoreCase(entry.getKey())) {
                res.add(entry.getValue());
            }
        }
        return res.toArray(new String[res.size()]);
    }

    /**
     * Gets the first header with the given name.
     * <p>
     * <p>
     * Header name comparison is case insensitive.
     *
     * @param name the name of the header to get
     * @return the first header or <code>null</code>
     */
    @Override
    public String getFirstHeader(String name) {
        if (name == null) {
            return null;
        }
        List<Map.Entry<String, String>> headers = mUnetResponse.getAllHeadersAsList();
        for (Map.Entry<String, String> entry : headers) {
            if (name.equalsIgnoreCase(entry.getKey())) {
                return entry.getValue();
            }
        }
        return null;
    }

    /**
     * Gets the last header with the given name.
     * <p>
     * <p>
     * Header name comparison is case insensitive.
     *
     * @param name the name of the header to get
     * @return the last header or <code>null</code>
     */
    @Override
    public String getLastHeader(String name) {
        return getFirstHeader(name);
    }

    /**
     * Gets a header representing all of the header values with the given name. If more that one header with the given
     * name exists the values will be combined with a "," as per RFC 2616.
     * <p>
     * <p>
     * Header name comparison is case insensitive.
     *
     * @param name the name of the header(s) to get
     * @return a header string array with a condensed value or <code>null</code> if no headers by the given name are
     * present
     */
    @Override
    public String getCondensedHeader(String name) {
        throw new UnsupportedOperationException("no impl in unet. ");

    }

    /**
     * get location, location will null when sync request , because sync request will redirect auto
     *
     * @return
     */
    @Override
    public String getLocation() {
        NetLog.e("unet","UnetResponseAdaptor getLocation");
        return mUnetResponse.getLastLocation();
    }

    /**
     * get last modified
     *
     * @return
     */
    @Override
    public String getLastModified() {
        return getFirstHeader(Headers.LAST_MODIFIED);
    }

    /**
     * get expires
     *
     * @return
     */
    @Override
    public String getExpires() {
        return getFirstHeader(Headers.EXPIRES);
    }

    /**
     * get etag
     *
     * @return
     */
    @Override
    public String getEtag() {
        return getFirstHeader(Headers.ETAG);
    }

    /**
     * get content encoding
     *
     * @return
     */
    @Override
    public String getContentEncoding() {
        return getFirstHeader(Headers.CONTENT_ENCODING);
    }

    /**
     * get content type
     *
     * @return
     */
    @Override
    public String getContentType() {
        return getFirstHeader(Headers.CONTENT_TYPE);
    }

    /**
     * get content length
     *
     * @return
     */
    @Override
    public long getContentLength() {
        try {
            return Long.parseLong(getFirstHeader(Headers.CONTENT_LENGTH));
        } catch (NumberFormatException e) {
            return Headers.NO_CONTENT_LENGTH;
        }
    }

    /**
     * get proxy authenticate
     *
     * @return
     */
    @Override
    public String getProxyAuthenticate() {
        return getFirstHeader(Headers.PROXY_AUTHENTICATE);

    }

    /**
     * get www authenticate
     *
     * @return
     */
    @Override
    public String getWwwAuthenticate() {
        return getFirstHeader(Headers.WWW_AUTHENTICATE);
    }

    /**
     * get transfer encoding
     *
     * @return
     */
    @Override
    public String getTransferEncoding() {
        return getFirstHeader(Headers.TRANSFER_ENCODING);
    }

    /**
     * get connection type
     *
     * @return
     */
    @Override
    public String getConnectionType() {
        return getFirstHeader(Headers.CONNECTION);
    }

    /**
     * get disposition
     *
     * @return
     */
    @Override
    public String getContentDisposition() {
        return getFirstHeader(Headers.CONTENT_DISPOSITION);
    }

    /**
     * get accept ranges
     *
     * @return
     */
    @Override
    public String getAcceptRanges() {
        return getFirstHeader(Headers.ACCEPT_RANGES);
    }

    /**
     * get cache control
     *
     * @return
     */
    @Override
    public String getCacheControl() {
        return getFirstHeader(Headers.CACHE_CONTROL);
    }

    /**
     * get cookies
     *
     * @return
     */
    @Override
    public String[] getCookies() {
        return getHeaders(Headers.COOKIE);

    }

    /**
     * get pragma
     *
     * @return
     */
    @Override
    public String getPragma() {
        return getFirstHeader(Headers.PRAGMA);
    }

    /**
     * get xpermitted cross domain policies
     *
     * @return
     */
    @Override
    public String getXPermittedCrossDomainPolicies() {
        return getFirstHeader(Headers.X_PERMITTED_CROSS_DOMAIN_POLICIES);
    }

    /**
     * response body
     */
    @Override
    public InputStream readResponse() throws IOException {
        return mUnetResponse.readResponse();
    }
}