/*
 * Copyright (C) 2004 - 2017 UCWeb Inc. All Rights Reserved.
 * Description : uPaaS Manager无线保镖加解密,sign接口适配
 *
 * Creation    :  2017-11-18
 * Author      : zhangxiang.zxi@alibaba-inc.com
 */
package com.uc.base.net.unet.util;


import android.content.Context;
import android.util.Log;

import com.alibaba.mbg.upaas.SecurityGuardWrapper;
import com.alibaba.wireless.security.open.SecException;
import com.alibaba.wireless.security.open.SecurityGuardManager;
import com.alibaba.wireless.security.open.SecurityGuardParamContext;
import com.alibaba.wireless.security.open.securesignature.ISecureSignatureComponent;
import com.alibaba.wireless.security.open.securesignature.SecureSignatureDefine;

import java.nio.ByteBuffer;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;


public class SecurityGuardWrapperImpl implements SecurityGuardWrapper {
    private static final String TAG = "SecurityGuardWrapper";

    // 无线保镖加密图片的名称后缀
    private String mWSGAuthCode;
    // 无线保镖加密app key
    private String mWSGAppKey;

    private Context mContext;
    private ISecureSignatureComponent mSignatureComponent;

    public SecurityGuardWrapperImpl(Context context, String appKey, String authCode) {
        mContext = context.getApplicationContext();
        mWSGAppKey = appKey;
        mWSGAuthCode = authCode;
    }

    @Override
    public String signRequest(String valueToSign, int requestType) {
        if (mSignatureComponent == null && mContext != null) {
            SecurityGuardManager instance;
            try {
                instance = SecurityGuardManager.getInstance(mContext);
            } catch (SecException e) {
                e.printStackTrace();
                return null;
            }
            if (instance != null) {
                mSignatureComponent = instance.getSecureSignatureComp();
                mContext = null;
            } else {
                return null;
            }
        }

        String key = mWSGAppKey;
        Map<String, String> paramMap = new HashMap<>(1);
        paramMap.put(SecureSignatureDefine.OPEN_KEY_SIGN_INPUT, valueToSign);
        SecurityGuardParamContext paramContext = new SecurityGuardParamContext();
        paramContext.appKey = key;
        paramContext.paramMap = paramMap;
        paramContext.requestType = requestType;
        String signature;
        try {
            signature = mSignatureComponent.signRequest(paramContext, mWSGAuthCode);
            byte[] bytes = ByteBuffer.allocate(2).putShort(Short.valueOf(key)).array();
            String prefix = byteToHexString(bytes);
            signature = prefix + signature;
        } catch (SecException e) {
            e.printStackTrace();
            return null;
        }
        Log.i(TAG, "signRequest,ori=" + valueToSign);
        Log.i(TAG, "signRequest,sign=" + signature);
        return signature;
    }

    private static String byteToHexString(byte[] bArr) {
        if (bArr == null || bArr.length <= 0) {
            return null;
        }
        StringBuffer stringBuffer = new StringBuffer(bArr.length * 2);
        int i = 0;
        while (i < bArr.length) {
            if ((bArr[i] & 255) < 16) {
                stringBuffer.append("0");
            }
            stringBuffer.append(Long.toString((long) (bArr[i] & 255), 16));
            i++;
        }
        return stringBuffer.toString().toLowerCase(Locale.ENGLISH);
    }
}
