/**  
 *****************************************************************************
 * Copyright (C) 2005-2014 UCWEB Corporation. All rights reserved
 * File        : 2014-3-3
 *
 * Description : 
 *
 * Creation    : 2014-3-3
 * Author      : linhz@ucweb.com
 * History     : Creation, 2014-3-3, linhz, Create the file
 *****************************************************************************
 */
package com.uc.base.net.util;

import android.text.TextUtils;

import java.io.UnsupportedEncodingException;
import java.math.BigInteger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Util {
    
    public static final int CR = 13; // <US-ASCII CR, carriage return (13)>
    public static final int LF = 10; // <US-ASCII LF, linefeed (10)>
    public static final int SP = 32; // <US-ASCII SP, space (32)>
    public static final int HT = 9; // <US-ASCII HT, horizontal-tab (9)>
    
    public static boolean isWhitespace(char ch) {
        return ch == SP || ch == HT || ch == CR || ch == LF;
    }

    public static long StringToLong(String value) {
        try {
            return Long.valueOf(value);
        } catch (Exception e) {
            return 0;
        }
    }

    /**
     *
     * //TODO luogw自动生成javadoc 需要后续其它同学补充注释
     * <p>请具体描述stringSplit方法提供的功能</p>
     *
     * <b>修改历史</b>
     * <ol>
     * <li>创建（Added by luogw on 2011-11-16）</li>
     * </ol>
     * @param original
     * @param regex
     * @return
     */
    public static String[] split(String original, String regex) {
        return split(original,regex,true);
    }

    /**
     *
     * //TODO luogw自动生成javadoc 需要后续其它同学补充注释
     * <p>请具体描述split方法提供的功能</p>
     *
     * <b>修改历史</b>
     * <ol>
     * <li>创建（Added by luogw on 2011-11-16）</li>
     * </ol>
     * @param aOriginal
     * @param aRegex
     * @param aCanNull
     * @return
     */
    public static String[] split(String aOriginal, String aRegex, boolean aCanNull) {
        if (TextUtils.isEmpty(aOriginal)) {
            return new String[0];
        }

        if( aRegex == null || aRegex.length() == 0){
            return  new String[]{ aOriginal };
        }

        String[] sTarget = null;
        int sTargetLength = 0;
        int sLength = aOriginal.length();
        int sStartIndex = 0;
        int sEndIndex = 0;

        //扫描字符串，确定目标字符串数组的长度
        for (sEndIndex = aOriginal.indexOf(aRegex, 0); sEndIndex != -1 && sEndIndex < sLength;
             sEndIndex = aOriginal.indexOf(aRegex, sEndIndex)) {
            sTargetLength += (aCanNull || sStartIndex != sEndIndex) ? 1 : 0;
            sStartIndex = sEndIndex += sEndIndex >= 0 ? aRegex.length() : 0;
        }

        //如果最后一个标记的位置非字符串的结尾，则需要处理结束串
        sTargetLength += aCanNull || sStartIndex != sLength ? 1 : 0;

        //重置变量值，根据标记拆分字符串
        sTarget = new String[sTargetLength];
        int sIndex = 0;
        for (sIndex = 0, sEndIndex = aOriginal.indexOf(aRegex, 0), sStartIndex = 0;
             sEndIndex != -1 && sEndIndex < sLength;
             sEndIndex = aOriginal.indexOf(aRegex, sEndIndex)) {
            if (aCanNull || sStartIndex != sEndIndex) {
                sTarget[sIndex] = aOriginal.substring(sStartIndex, sEndIndex);
                ++sIndex;
            }
            sStartIndex = sEndIndex += sEndIndex >= 0 ? aRegex.length() : 0;
        }

        //取结束的子串
        if (aCanNull || sStartIndex != sLength) {
            sTarget[sTargetLength - 1] = aOriginal.substring(sStartIndex); ;
        }

        return sTarget;
    }

    /**
     * @return 是否包含中文
     */
    public static boolean containsChinese(String str) {
        try{
            Pattern p = Pattern.compile("[\\u4e00-\\u9fa5]");
            Matcher m = p.matcher(str);
            if (m.find()) {
                return true;
            }
        }catch(Exception e){
            //ExceptionHandler.processFatalException(e);
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 转换字符串中非ascii码字符为utf-8编码的16进制数字格式
     * @param value 需要转换的字符串
     * @return 转换后的字符串
     */
    public static String urlEncodeKeepAscii(String value)
    {
        if (value == null)
            return null;
        int length = value.length();
        if (length == 0)
            return "";
        StringBuilder asciiString = new StringBuilder();
        for (int i=0; i<length; i++)
        {
            char c = value.charAt(i);
            if (c <0 || c>127)
            {
                try
                {
                    String hexRaw = String.format("%x", new BigInteger(1, value.substring(i,i+1).getBytes("UTF-8")));
                    char[] hexRawArr = hexRaw.toCharArray();
                    final String SEP = "%";
                    for (int j = 0; j < hexRawArr.length; j++) {
                        asciiString.append(SEP).append(hexRawArr[j]).append(hexRawArr[++j]);
                    }
                }
                catch (UnsupportedEncodingException e)
                {
                }
            }
            else
                asciiString.append(c);
        }
        return asciiString.toString();
    }


}
